%Function:  Import the raw data and put them into a big array
%Param:     dataFileName_       The text file containing the raw data
%           timeGap_            The time gap used to filter the data
%           flagDrawFigure_     0, don't draw figure in the function
%                               not 0, draw it
%Return:    indoor_temp_array       The indoor temperature data array
%           outdoor_temp_array      The outdoor temperature data array
%           energy_array            The time array
%           
%Array format: time, node1~node10, node15
function [indoor_temp_array, outdoor_temp_array, energy_array]=Fn_ImportEnergyPlusData(dataFileName_, timeGap_, flagDrawFigure_)
TIME_ALIGN_SIZE = 1000*60;   %Align at 1 min

%Import real data
fid = fopen(dataFileName_);
if(fid == -1)
    Aligned_array_size = -1;
    fprintf('Cannot open file: %s\n',dataFileName_);
    return;
end
C = textscan(fid, '%f %f %f %f');
fclose(fid);

outdoor_temp_array = C{1}';
heating_array = C{2}';
cooling_array = C{3}';
indoor_temp_array = C{4}';
energy_array = cooling_array - heating_array;


if flagDrawFigure_ == 0
    return;
end

array_size = size(energy_array, 2);
Style_arr = {'-k','--k',':k','-b','--b',':b','-m','--m',':m','-r' };
t = 1:timeGap_:array_size*timeGap_;
hold on

plot(t,indoor_temp_array, '-b');
plot(t, outdoor_temp_array, '-r')

hold off

end
