
#include "GroupInfoManager.h"
#include "DPScheduleAlg.h"

MeetingInfo_ptr GroupInfoManager::addMeetingInfo(MeetingInfo& meetingInfo)
{
	for(int i= 0; i<m_rawMeetingNumber; i++)
	{
		if(m_rawMeetingArr[i].isSameMeeting(&meetingInfo))
			return &m_rawMeetingArr[i];
	}

	if(m_rawMeetingNumber>= MAX_MEETING_NUMBER)
		return NULL;
	
	m_rawMeetingArr[m_rawMeetingNumber]=meetingInfo;
	m_meetingArr[m_meetingNumber] = &m_rawMeetingArr[m_rawMeetingNumber];
	m_rawMeetingNumber++;
	m_meetingNumber++;

	
	return &m_rawMeetingArr[m_rawMeetingNumber-1];
}

MeetingInfoNum GroupInfoManager::addUnscheduledMeeting(MeetingInfo_ptr meetingInfo)
{
	// TODO: Same meeting check

	//Reset the dummy schedule 
	meetingInfo->resetSchedule();
	m_meetingArr[m_meetingNumber] = meetingInfo;
	m_meetingNumber++;
	return 0;
}


MeetingInfo_ptr GroupInfoManager::getMeetingInfo(MeetingInfoNum meetingNum)
{
	if(meetingNum >= m_meetingNumber)
		return NULL;

	return m_meetingArr[meetingNum];
}

RoomInfo_ptr GroupInfoManager::addRoomInfo(RoomInfo& roomInfo)
{
	for(int i= 0; i<m_roomNumber; i++)
	{
		if(m_roomArr[i].isSameRoom(roomInfo))
			return &m_roomArr[i];
	}

	if(m_roomNumber>= MAX_MEETING_NUMBER)
		return NULL;
	
	m_roomArr[m_roomNumber]=roomInfo;
	m_roomNumber++;
	return &m_roomArr[m_roomNumber-1];
}

RoomInfo_ptr GroupInfoManager::getRoomInfo(RoomInfoNum roomNum)
{
	if(roomNum >= m_roomNumber)
		return NULL;

	return &m_roomArr[roomNum];
}
RoomInfo_ptr GroupInfoManager::getRoomInfoByName(const char * roomName)
{
	RoomInfo_ptr room_info = NULL;
	for(int i=0; i< m_roomNumber; i++)
	{
		room_info = &m_roomArr[i];
		if((strcmp(roomName, room_info->roomName()))==0)
			return room_info;
	}
	return NULL;
}

int GroupInfoManager::sort()
{
	int i, j;
	MeetingInfo_ptr temp;
	for(i = 0; i < m_meetingNumber; i++)
	{
		m_startTimeArr[i] = m_meetingArr[i];
		m_endTimeArr[i]   = m_meetingArr[i];
	}
	
	//Accending sort by the start time
    for( i = 0; i < m_meetingNumber; i++) {
        for( j = i; j < m_meetingNumber; j++) {
			if( m_startTimeArr[j]->startTime()< m_startTimeArr[i]->startTime() ) {
				//Exchange the i and j;
                temp = m_startTimeArr[j];
                m_startTimeArr[j] = m_startTimeArr[i];
                m_startTimeArr[i] = temp;
            }
        }
    }

	//Accending sort by the end time
    for( i = 0; i < m_meetingNumber; i++) {
        for( j = i; j < m_meetingNumber; j++) {
			if( m_endTimeArr[j]->endTime()< m_endTimeArr[i]->endTime() ) {
				//Exchange the i and j;
                temp = m_endTimeArr[j];
                m_endTimeArr[j] = m_endTimeArr[i];
                m_endTimeArr[i] = temp;
            }
        }
    }


	// Rooms are  sorted by facilities in descending order
	for(i= 0; i<m_roomNumber; i++)
	{
		m_roomFacilityArr[i] = &m_roomArr[i];
	}
	for(i = 0; i< m_roomNumber; i++)
	{
		for(j = i; j<m_roomNumber; j++)
		{
			if(m_roomFacilityArr[j]->facility() > m_roomFacilityArr[i]->facility())
			{
				//Exchange i and j
                RoomInfo_ptr temp_room = m_roomFacilityArr[j];
                m_roomFacilityArr[j] = m_roomFacilityArr[i];
                m_roomFacilityArr[i] = temp_room;
			}
		}
	}
	return 1;

}

int GroupInfoManager::recoverRawSchedule()
{
	int ret;
	RoomInfo_ptr room_info;
	MeetingInfo_ptr current_meeting;
	int scheduled_num = 0;
	
	for(int i= 0; i< m_meetingNumber; i++)
	{
		current_meeting = m_endTimeArr[i];
		room_info = getRoomInfoByName(current_meeting->roomName());
		//No room found? should in other capacity group
		if(room_info == NULL)
		{
			m_unschedulabeMeetingArr[m_unschedulableMeetingNumber]=current_meeting;
			current_meeting->room(&m_nullRoom);	//dummy schedule
			m_unschedulableMeetingNumber++;
		}
		else
		{
			ret =room_info->putMeeting(current_meeting);
			scheduled_num++;
		}
	}
	return scheduled_num;
}


int GroupInfoManager::optimalSchedule()
{
	int scheduled_start = -1;
	int scheduled_end_time = 0;
	MeetingInfo_ptr schedulable_group[MAX_MEETING_NUMBER];


	while(1){
		int schedulable_meeting_num=0;

		//select the schedulable group
		for(int i = 0; i<m_meetingNumber; i++)
		{
			int flag_schedulable = 1; 
			MeetingInfo_ptr current = m_endTimeArr[i];
			if(current->isScheduled())
				continue;

			for(int j = 0; j < schedulable_meeting_num; j++)
			{
				//As we are searching the list according to the end time
				if( schedulable_group[j]->endTime()<= current->startTime())
				{
					flag_schedulable = 0;
					break;
				}
			}

			if(flag_schedulable)
			{
				schedulable_group[schedulable_meeting_num] = current;
				schedulable_meeting_num++;
			}
		}//end select schedulable group


		if(schedulable_meeting_num == 0 )
			break;
		
		//Schedule the schedulable meetings.
		MeetingInfo_ptr meeting_save_most;
		RoomInfo_ptr room_save_most= NULL;	//Maybe no room fix
		int time_minimum_value = 2000;
		
		for(int i= 0; i<schedulable_meeting_num; i++)
		{
			int meeting_start_time = schedulable_group[i]->startTime();
			for(int j = 0; j<m_roomNumber; j++)
			{
				int room_end_time = m_roomArr[j].lastMeetingEndTime();
				int time_interval;
				//Schedulable?
				if(room_end_time > meeting_start_time)
					continue;

				time_interval = meeting_start_time - room_end_time;
				if(time_interval < time_minimum_value)
				{
					time_minimum_value = time_interval;
					meeting_save_most = schedulable_group[i];
					room_save_most = &m_roomArr[j];
				}
			}
		}

		//All the meetings are unschedulable?
		if(room_save_most == NULL)
		{
			for(int i= 0; i<schedulable_meeting_num; i++)
			{
				m_unschedulabeMeetingArr[m_unschedulableMeetingNumber]=schedulable_group[i];
				schedulable_group[i]->room(&m_nullRoom);	//dummy schedule
				m_unschedulableMeetingNumber++;
			}
		}
		else
		{
			room_save_most->putMeeting(meeting_save_most);
		}
	
	}
	

	return 0;
}


int GroupInfoManager::energyAwareFacilitySchedule()
{
	if(m_meetingNumber == 0)
		return 0;
	// The meetings are sorted according to their start time

	//In the first round, fit meetings that have facility requirement
	for(int i = 0; i< m_meetingNumber; i++)
	{
		int room_num = -1;	//Record the matched room num
		int best_interval_time = 20000;
		MeetingInfo_ptr current_meeting = m_startTimeArr[i];
		int cm_start_time = current_meeting->startTime();	//Current meeting start time
		if(current_meeting->isScheduled())
			continue;

		//Only schedule meetings have facility requirement
		if(current_meeting->facilityRequirement()==0)
			continue;
		
		for(int j = 0; j < m_roomNumber; j++)
		{
			int room_last_end_time = m_roomFacilityArr[j]->lastMeetingEndTime();
			//If can not be filled
			if(room_last_end_time > cm_start_time)
				continue;

			if((cm_start_time-room_last_end_time) < best_interval_time)
			{
				best_interval_time = cm_start_time - room_last_end_time;
				room_num = j;
			}
		}

		if(room_num != -1)
		{
			m_roomFacilityArr[room_num]->putMeeting(current_meeting);
		}
		else
		{
			
			m_unschedulabeMeetingArr[m_unschedulableMeetingNumber]=current_meeting;
			current_meeting->room(&m_nullRoom);	//dummy schedule
			m_unschedulableMeetingNumber++;
		}
	}
	
	//In the second round, fit meeting withourt facility requirement
	for(int i = 0; i< m_meetingNumber; i++)
	{
		int room_num = -1;	//Record the matched room num
		int best_interval_time = 20000;
		MeetingInfo_ptr current_meeting = m_startTimeArr[i];
		int cm_start_time = current_meeting->startTime();	//Current meeting start time
		if(current_meeting->isScheduled())
			continue;

		//Try insert first
		for(int j = 0; j < m_roomNumber; j++)
		{
			if(0==m_roomFacilityArr[j]->insertMeeting(current_meeting))
				break;
		}

		if(current_meeting->isScheduled())
			continue;
		
		for(int j = 0; j < m_roomNumber; j++)
		{
			int room_last_end_time = m_roomFacilityArr[j]->lastMeetingEndTime();
			//If can not be filled
			if(room_last_end_time > cm_start_time)
				continue;

			if((cm_start_time-room_last_end_time) < best_interval_time)
			{
				best_interval_time = cm_start_time - room_last_end_time;
				room_num = j;
			}
		}

		if(room_num != -1)
		{
			m_roomFacilityArr[room_num]->putMeeting(current_meeting);
		}
		else
		{
			
			m_unschedulabeMeetingArr[m_unschedulableMeetingNumber]=current_meeting;
			current_meeting->room(&m_nullRoom);	//dummy schedule
			m_unschedulableMeetingNumber++;
		}
	}	

	
	return 0;
}

int GroupInfoManager::bestFitSchedule()
{
	for(int i= 0; i< m_meetingNumber; i++)
	{
		int room_num = -1;
		int min_meeting_interval = 2000;
		MeetingInfo_ptr current_meeting = m_endTimeArr[i];
		int meeting_start_time = current_meeting->startTime();

		for(int j=0; j< m_roomNumber; j++)
		{
			int end_time = m_roomArr[j].lastMeetingEndTime();
			int meeting_interval;
			//Schedulable?
			if(end_time > meeting_start_time)
				continue;
			meeting_interval = meeting_start_time - end_time;
			if(meeting_interval< min_meeting_interval)
			{
				room_num = j;
				min_meeting_interval = meeting_interval;
			}
			
		}
		if(room_num != -1)
		{
			m_roomArr[room_num].putMeeting(current_meeting);
		}
		else
		{
			
			m_unschedulabeMeetingArr[m_unschedulableMeetingNumber]=current_meeting;
			current_meeting->room(&m_nullRoom);	//dummy schedule
			m_unschedulableMeetingNumber++;
		}
	}

	return 0;
}

int GroupInfoManager::timeUnrestrictedSchedule()
{
	optimalSchedule();

	for(int i = 0; i<m_roomNumber; i++)
	{
		m_roomFacilityArr[i]->compressSchedule();
	}
	return 0;
}

int GroupInfoManager::timeUnrestrictedFacilitySchedule()
{
	energyAwareFacilitySchedule();

	for(int i = 0; i<m_roomNumber; i++)
	{
		m_roomFacilityArr[i]->compressSchedule();
	}
	return 0;

}

int GroupInfoManager::resetSchedule()
{
	for(int i=0; i<m_roomNumber; i++)
	{
		m_roomArr[i].resetSchedule();
	}

	//recover m_meetingArr
	for(int i= 0; i<m_meetingNumber; i++)
	{
		if(i<m_rawMeetingNumber)
		{
			m_meetingArr[i] = &m_rawMeetingArr[i];
		}
		else
		{
			m_meetingArr[i] = NULL;
		}
	}
	m_meetingNumber = m_rawMeetingNumber;
	
	m_unschedulableMeetingNumber = 0;
	
	return 0;
}

float GroupInfoManager::calculateEnergy()
{
	float energy_consumption = 0;
	for(int i= 0; i<m_roomNumber; i++)
	{
		energy_consumption += m_roomArr[i].calculateRoomEnergy();
	}
	return energy_consumption;
}

float GroupInfoManager::calculateMinEnergy()
{
	float energy_consumption = 0;
	for(int i= 0; i<m_roomNumber; i++)
	{
		energy_consumption += m_roomArr[i].calculateRoomMinEnergy();
	}
	return energy_consumption;
}
float GroupInfoManager::calculateMaxEnergy()
{
	float energy_consumption = 0;
	for(int i= 0; i<m_roomNumber; i++)
	{
		energy_consumption += m_roomArr[i].calculateRoomMaxEnergy();
	}
	return energy_consumption;
}

float GroupInfoManager::generateEnergyPlusInput(float* originArr, FILE* fpZone, FILE* fpSurface, FILE* fpHVAC, FILE* fpZoneHVACmap, FILE* fpSchedule, FILE* fpScheduleDummy, FILE* fpScheduleWOR)
{
	float x = originArr[0];
	float y = originArr[1];
	float z = originArr[2];

	if(m_roomNumber == 0)
		return 0;

	for(int i=0; i<m_roomNumber; i++)
	{
		x= x+m_roomFacilityArr[i]->length();
		m_roomFacilityArr[i]->setZoneOriginal(x,y,z);
		m_roomFacilityArr[i]->generateZoneData(fpZone);
		m_roomFacilityArr[i]->generateSurfaces(fpSurface);
		m_roomFacilityArr[i]->generateHVAC(fpHVAC);
		m_roomFacilityArr[i]->generateZoneHVAC(fpZoneHVACmap);
		m_roomFacilityArr[i]->generateHVACSchedule(fpSchedule, SCHEDULE_PURE);
		m_roomFacilityArr[i]->generateHVACSchedule(fpScheduleDummy, SCHEDULE_DUMMY);
		m_roomFacilityArr[i]->generateHVACSchedule(fpScheduleWOR, SCHEDULE_WITHOUT_RECOOL);
	}
	return 0;
}
