
#include "MeetingInfo.h"


/*Zone,
    ZONE1,                   !- Name
    0,                       !- Direction of Relative North {deg}
    0,                       !- X Origin {m}
    0,                       !- Y Origin {m}
    0,                       !- Z Origin {m}
    1,                       !- Type
    1,                       !- Multiplier
    2.7000,                  !- Ceiling Height {m}
    129.6;                   !- Volume {m3}
*/
int RoomInfo::generateZoneData(FILE* fp)
{
fprintf(fp, "Zone,\n");
	fprintf(fp, "\t%s,\n", m_roomName);
	fprintf(fp, "\t0,\n\t0,\n\t0,\n\t0,\n\t1,\n\t1,\n");
	fprintf(fp, "\t%f,\n", m_height);
	fprintf(fp, "\t%f;\n\n\n", m_height*m_length*m_width);
    return 0;	
}

/*
BuildingSurface:Detailed,
    Z1 NORTH,                !- Name
    Wall,                    !- Surface Type
    LTWALL,                  !- Construction Name
    ZONE1,                   !- Zone Name
    Outdoors,                !- Outside Boundary Condition
    ,                        !- Outside Boundary Condition Object
    SunExposed,              !- Sun Exposure
    WindExposed,             !- Wind Exposure
    0.50,                    !- View Factor to Ground
    4,                       !- Number of Vertices
    8.00,                    !- Vertex 1 X-coordinate {m}
    6.00,                    !- Vertex 1 Y-coordinate {m}
    2.70,                    !- Vertex 1 Z-coordinate {m}
    8.00,                    !- Vertex 2 X-coordinate {m}
    6.00,                    !- Vertex 2 Y-coordinate {m}
    0,                       !- Vertex 2 Z-coordinate {m}
    0,                       !- Vertex 3 X-coordinate {m}
    6.00,                    !- Vertex 3 Y-coordinate {m}
    0,                       !- Vertex 3 Z-coordinate {m}
    0,                       !- Vertex 4 X-coordinate {m}
    6.00,                    !- Vertex 4 Y-coordinate {m}
    2.70;                    !- Vertex 4 Z-coordinate {m}

*/
int RoomInfo::generateSurfaces(FILE* fp)
{
	const char* n_arr[6] =  {"North", 			"South", 		"East", 		"West", 		"Roof", 		"Floor"};	//Name
	const char* sn_arr[6] =  {"Wall", 			"Wall", 		"Wall", 		"Wall", 		"Roof", 		"Floor"};	//Surface name
	const char* cn_arr[6] =  {"LTWALL",			"LTWALL", 		"LTWALL",		"LTWALL",		"LTROOF",		"LTFLOOR"};	//construction name
	const char* obc_arr[6] = {"Outdoors",		"Outdoors", 	"Outdoors",		"Outdoors",		"Outdoors",		"Ground"};	//Outside Boundary Condition
	const char* se_arr[6] = {"SunExposed",		"SunExposed", 	"SunExposed",	"SunExposed",	"SunExposed",	"NoSun"};	//Sun Exposure
	const char* we_arr[6] = {"WindExposed",		"WindExposed", 	"WindExposed",	"WindExposed",	"WindExposed",	"NoWind"};	//Wind Exposure
	const char* vfg_arr[6] = {"0.5",			"0.5", 			"0.5",			"0.5",			"0",			"0"};	//View Factor to Ground

	float x_arr[4][6];
	float y_arr[4][6];
	float z_arr[4][6];

	// Length: x, width: y, height: z
	//Y is North axis
	//GlobalGeometryRules,
	//	UpperLeftCorner,		 !- Starting Vertex Position
	//	Counterclockwise,		 !- Vertex Entry Direction
	//	WorldCoordinateSystem;	 !- Coordinate System
	
	//North surface
	x_arr[0][0] = m_x+m_length;	y_arr[0][0]=m_y+m_width;			z_arr[0][0]=m_z + m_height;
	x_arr[1][0] = m_x+m_length;	y_arr[1][0]=m_y+m_width;			z_arr[1][0]=m_z;
	x_arr[2][0] = m_x;				y_arr[2][0]=m_y+m_width;			z_arr[2][0]=m_z;
	x_arr[3][0] = m_x;				y_arr[3][0]=m_y+m_width;			z_arr[3][0]=m_z + m_height;

	//South surface
	x_arr[0][1] = m_x;				y_arr[0][1]=m_y;					z_arr[0][1]=m_z + m_height;
	x_arr[1][1] = m_x;				y_arr[1][1]=m_y;					z_arr[1][1]=m_z;
	x_arr[2][1] = m_x+m_length;	y_arr[2][1]=m_y;					z_arr[2][1]=m_z;
	x_arr[3][1] = m_x+m_length;	y_arr[3][1]=m_y;					z_arr[3][1]=m_z + m_height;

	//East surface
	x_arr[0][2] = m_x+m_length;	y_arr[0][2]=m_y;					z_arr[0][2]=m_z + m_height;
	x_arr[1][2] = m_x+m_length;	y_arr[1][2]=m_y;					z_arr[1][2]=m_z;
	x_arr[2][2] = m_x+m_length;	y_arr[2][2]=m_y+m_width;			z_arr[2][2]=m_z;
	x_arr[3][2] = m_x+m_length;	y_arr[3][2]=m_y+m_width;			z_arr[3][2]=m_z + m_height;

	//West surface
	x_arr[0][3] = m_x;				y_arr[0][3]=m_y+m_width;			z_arr[0][3]=m_z + m_height;
	x_arr[1][3] = m_x;				y_arr[1][3]=m_y+m_width;			z_arr[1][3]=m_z;
	x_arr[2][3] = m_x;				y_arr[2][3]=m_y;					z_arr[2][3]=m_z;
	x_arr[3][3] = m_x;				y_arr[3][3]=m_y;					z_arr[3][3]=m_z + m_height;


	//Roof surface
	x_arr[0][4] = m_x;				y_arr[0][4]=m_y;					z_arr[0][4]=m_z + m_height;
	x_arr[1][4] = m_x+m_length;	y_arr[1][4]=m_y;					z_arr[1][4]=m_z + m_height;
	x_arr[2][4] = m_x+m_length;	y_arr[2][4]=m_y+m_width;			z_arr[2][4]=m_z + m_height;
	x_arr[3][4] = m_x;				y_arr[3][4]=m_y+m_width;			z_arr[3][4]=m_z + m_height;

	//Floor surface
	x_arr[0][5] = m_x;				y_arr[0][5]=m_y;					z_arr[0][5]=m_z;
	x_arr[1][5] = m_x;				y_arr[1][5]=m_y+m_width;			z_arr[1][5]=m_z;
	x_arr[2][5] = m_x+m_length;	y_arr[2][5]=m_y+m_width;			z_arr[2][5]=m_z;
	x_arr[3][5] = m_x+m_length;	y_arr[3][5]=m_y;					z_arr[3][5]=m_z;


	for(int i = 0; i<6; i++)
	{
	fprintf(fp, "BuildingSurface:Detailed,\n");
		fprintf(fp, "\t%s %s,\n", m_roomName, n_arr[i]);	//Z1 NORTH,                !- Name
		fprintf(fp, "\t%s,\n", sn_arr[i]);    		//Wall,                    !- Surface Type
	    fprintf(fp, "\t%s,\n", cn_arr[i]);			//LTWALL,                  !- Construction Name
	    fprintf(fp, "\t%s,\n", m_roomName);    	//ZONE1,                   !- Zone Name
	    fprintf(fp, "\t%s,\n", obc_arr[i]);    	//Outdoors,                !- Outside Boundary Condition
	    fprintf(fp, "\t ,\n");    					//,                        !- Outside Boundary Condition Object
	    fprintf(fp, "\t%s,\n", se_arr[i]);    		//SunExposed,              !- Sun Exposure
	    fprintf(fp, "\t%s,\n", we_arr[i]);    		//WindExposed,             !- Wind Exposure
	    fprintf(fp, "\t%s,\n", vfg_arr[i]);    	//0.50,                    !- View Factor to Ground
	    fprintf(fp, "\t 4,\n");    					//4,                       !- Number of Vertices

		fprintf(fp, "\t%.2f,\n", x_arr[0][i]);    	//8.00,                    !- Vertex 1 X-coordinate {m}
	    fprintf(fp, "\t%.2f,\n", y_arr[0][i]);		//6.00,                    !- Vertex 1 Y-coordinate {m}
	    fprintf(fp, "\t%.2f,\n", z_arr[0][i]);		//2.70,                    !- Vertex 1 Z-coordinate {m}

		fprintf(fp, "\t%.2f,\n", x_arr[1][i]);    	//8.00,                    !- Vertex 2 X-coordinate {m}
	    fprintf(fp, "\t%.2f,\n", y_arr[1][i]);    	//6.00,                    !- Vertex 2 Y-coordinate {m}
	    fprintf(fp, "\t%.2f,\n", z_arr[1][i]);    	//0,                       !- Vertex 2 Z-coordinate {m}

		fprintf(fp, "\t%.2f,\n", x_arr[2][i]);    	//0,                       !- Vertex 3 X-coordinate {m}
	    fprintf(fp, "\t%.2f,\n", y_arr[2][i]);    	//6.00,                    !- Vertex 3 Y-coordinate {m}
	    fprintf(fp, "\t%.2f,\n", z_arr[2][i]);  		//0,                       !- Vertex 3 Z-coordinate {m}
	    
	    fprintf(fp, "\t%.2f,\n", x_arr[3][i]);    	//0,                       !- Vertex 4 X-coordinate {m}
	    fprintf(fp, "\t%.2f,\n", y_arr[3][i]);    	//6.00,                    !- Vertex 4 Y-coordinate {m}
	    fprintf(fp, "\t%.2f;\n", z_arr[3][i]);    	//2.70;                    !- Vertex 4 Z-coordinate {m}
	    fprintf(fp, "\n\n");
	}    
    return 0;
	
}

/*
HVACTemplate:System:Unitary,
    HVAC system 1,           !- Name
    Office HVAC Mixed,       !- System Availability Schedule Name
    ZONE1,                   !- Control Zone or Thermostat Location Name
    autosize,                !- Supply Fan Maximum Flow Rate {m3/s}
    Constant,                !- Supply Fan Operating Mode Schedule Name
    0.7,                     !- Supply Fan Total Efficiency
    600,                     !- Supply Fan Delta Pressure {Pa}
    0.9,                     !- Supply Fan Motor Efficiency
    1,                       !- Supply Fan Motor in Air Stream Fraction
    SingleSpeedDX,           !- Cooling Coil Type
    ,                        !- Cooling Coil Availability Schedule Name
    12.8,                    !- Cooling Design Supply Air Temperature
    autosize,                !- Cooling Coil Capacity {W}
    autosize,                !- Cooling Coil Rated Sensible Heat Ratio
    3,                       !- Cooling Coil Rated COP {W/W}
    Electric,                !- Heating Coil Type
    ,                        !- Heating Coil Availability Schedule Name
    50,                      !- Heating Design Supply Air Temperature
    autosize,                !- Heating Coil Capacity {W}
    0.8,                     !- Gas Heating Coil Efficiency
    ,                        !- Gas Heating Coil Parasitic Electric Load {W}
    autosize,                !- Maximum Outdoor Air Flow Rate {m3/s}
    autosize,                !- Minimum Outdoor Air Flow Rate {m3/s}
    ,                        !- Minimum Outdoor Air Schedule Name
    NoEconomizer,            !- Economizer Type
    NoLockout,               !- Economizer Lockout
    ,                        !- Economizer Upper Temperature Limit {C}
    ,                        !- Economizer Lower Temperature Limit {C}
    ,                        !- Economizer Upper Enthalpy Limit {J/kg}
    ,                        !- Economizer Maximum Limit Dewpoint Temperature {C}
    ,                        !- Supply Plenum Name
    ,                        !- Return Plenum Name
    BlowThrough,             !- Supply Fan Placement
    StayOff,                 !- Night Cycle Control
    ,                        !- Night Cycle Control Zone Name
    None,                    !- Heat Recovery Type
    0.7,                     !- Sensible Heat Recovery Effectiveness
    0.65,                    !- Latent Heat Recovery Effectiveness
    None,                    !- Dehumidification Control Type
    ,                        !- Dehumidification Control Zone Name
    60,                      !- Dehumidification Setpoint {percent}
    None,                    !- Humidifier Type
    ,                        !- Humidifier Availability Schedule Name
    0.000001,                !- Humidifier Rated Capacity {m3/s}
    2690,                    !- Humidifier Rated Electric Power {W}
    ,                        !- Humidifier Control Zone Name
    30;                      !- Humidifier Setpoint {percent}

*/
int RoomInfo::generateHVAC(FILE* fp)
{
	fprintf(fp, "HVACTemplate:System:Unitary,\n"); //HVACTemplate:System:Unitary,
		fprintf(fp, "\t HVAC_%s,\n", m_roomName);//HVAC system 1,			 !- Name
		fprintf(fp, "\t HVAC_Schedule_%s,\n", m_roomName);//Office HVAC Mixed,		 !- System Availability Schedule Name
		fprintf(fp, "\t %s,\n", m_roomName);//ZONE1,					 !- Control Zone or Thermostat Location Name
		fprintf(fp, "\t autosize,\n"); //autosize,				 !- Supply Fan Maximum Flow Rate {m3/s}
		fprintf(fp, "\t HVAC_Schedule_%s,\n", m_roomName); //Office HVAC Mixed,				 !- Supply Fan Operating Mode Schedule Name
		fprintf(fp, "\t 0.7,\n"); //0.7,					 !- Supply Fan Total Efficiency
		fprintf(fp, "\t 600,\n"); //600,					 !- Supply Fan Delta Pressure {Pa}
		fprintf(fp, "\t 0.9,\n"); //0.9,					 !- Supply Fan Motor Efficiency
		fprintf(fp, "\t 1,\n"); //1,						 !- Supply Fan Motor in Air Stream Fraction
		fprintf(fp, "\t SingleSpeedDX,\n"); //SingleSpeedDX,			 !- Cooling Coil Type
		fprintf(fp, "\t HVAC_Schedule_%s,\n", m_roomName); //Office HVAC Mixed,						 !- Cooling Coil Availability Schedule Name
		fprintf(fp, "\t 12.8,\n"); //12.8,					 !- Cooling Design Supply Air Temperature
		fprintf(fp, "\t autosize,\n"); //autosize,				 !- Cooling Coil Capacity {W}
		fprintf(fp, "\t autosize,\n"); //autosize,				 !- Cooling Coil Rated Sensible Heat Ratio
		fprintf(fp, "\t 3,\n"); //3,						 !- Cooling Coil Rated COP {W/W}
		fprintf(fp, "\t Electric,\n"); //Electric,				 !- Heating Coil Type
		fprintf(fp, "\t HVAC_Schedule_%s,\n", m_roomName); //Office HVAC Mixed,						 !- Heating Coil Availability Schedule Name
		fprintf(fp, "\t 50,\n"); //50, 					 !- Heating Design Supply Air Temperature
		fprintf(fp, "\t autosize,\n"); //autosize,				 !- Heating Coil Capacity {W}
		fprintf(fp, "\t 0.8,\n"); //0.8,					 !- Gas Heating Coil Efficiency
		fprintf(fp, "\t ,\n"); //,						 !- Gas Heating Coil Parasitic Electric Load {W}
		fprintf(fp, "\t autosize,\n"); //autosize,				 !- Maximum Outdoor Air Flow Rate {m3/s}
		fprintf(fp, "\t autosize,\n"); //autosize,				 !- Minimum Outdoor Air Flow Rate {m3/s}
		fprintf(fp, "\t ,\n"); //,						 !- Minimum Outdoor Air Schedule Name
		fprintf(fp, "\t NoEconomizer,\n"); //NoEconomizer,			 !- Economizer Type
		fprintf(fp, "\t NoLockout,\n"); //NoLockout,				 !- Economizer Lockout
		fprintf(fp, "\t ,\n"); //,						 !- Economizer Upper Temperature Limit {C}
		fprintf(fp, "\t ,\n"); //,						 !- Economizer Lower Temperature Limit {C}
		fprintf(fp, "\t ,\n"); //,						 !- Economizer Upper Enthalpy Limit {J/kg}
		fprintf(fp, "\t ,\n"); //,						 !- Economizer Maximum Limit Dewpoint Temperature {C}
		fprintf(fp, "\t ,\n"); //,						 !- Supply Plenum Name
		fprintf(fp, "\t ,\n"); //,						 !- Return Plenum Name
		fprintf(fp, "\t BlowThrough,\n"); //BlowThrough,			 !- Supply Fan Placement
		fprintf(fp, "\t StayOff,\n"); //StayOff,				 !- Night Cycle Control
		fprintf(fp, "\t ,\n"); //,						 !- Night Cycle Control Zone Name
		fprintf(fp, "\t None,\n"); //None,					 !- Heat Recovery Type
		fprintf(fp, "\t 0.7,\n"); //0.7,					 !- Sensible Heat Recovery Effectiveness
		fprintf(fp, "\t 0.65,\n"); //0.65,					 !- Latent Heat Recovery Effectiveness
		fprintf(fp, "\t None,\n"); //None,					 !- Dehumidification Control Type
		fprintf(fp, "\t ,\n"); //,						 !- Dehumidification Control Zone Name
		fprintf(fp, "\t 60,\n"); //60, 					 !- Dehumidification Setpoint {percent}
		fprintf(fp, "\t None,\n"); //None,					 !- Humidifier Type
		fprintf(fp, "\t ,\n"); //,						 !- Humidifier Availability Schedule Name
		fprintf(fp, "\t 0.000001,\n"); //0.000001,				 !- Humidifier Rated Capacity {m3/s}
		fprintf(fp, "\t 2690,\n"); //2690,					 !- Humidifier Rated Electric Power {W}
		fprintf(fp, "\t ,\n"); //,						 !- Humidifier Control Zone Name
		fprintf(fp, "\t 30;\n"); //30; 					 !- Humidifier Setpoint {percent}
	fprintf(fp,"\n\n");

    return 0;

}

/*
HVACTemplate:Zone:Unitary,
    ZONE1,                   !- Zone Name
    HVAC system 1,           !- Template Unitary System Name
    HVAC thermostate,        !- Template Thermostat Name
    autosize,                !- Supply Air Maximum Flow Rate {m3/s}
    ,                        !- Zone Heating Sizing Factor
    ,                        !- Zone Cooling Sizing Factor
    Flow/Person,             !- Outdoor Air Method
    0.00944,                 !- Outdoor Air Flow Rate per Person {m3/s}
    ,                        !- Outdoor Air Flow Rate per Zone Floor Area {m3/s-m2}
    ,                        !- Outdoor Air Flow Rate per Zone {m3/s}
    ,                        !- Supply Plenum Name
    ,                        !- Return Plenum Name
    None,                    !- Baseboard Heating Type
    ,                        !- Baseboard Heating Availability Schedule Name
    autosize;                !- Baseboard Heating Capacity {W}

*/
int RoomInfo::generateZoneHVAC(FILE* fp)
{

fprintf(fp, "HVACTemplate:Zone:Unitary,\n");//HVACTemplate:Zone:Unitary,
	fprintf(fp, "\t %s,\n", m_roomName);//	ZONE1,					 !- Zone Name
	fprintf(fp, "\t HVAC_%s,\n", m_roomName);//	HVAC system 1,			 !- Template Unitary System Name
	fprintf(fp, "\t HVAC thermostate,\n");//	HVAC thermostate,		 !- Template Thermostat Name
	fprintf(fp, "\t autosize,\n");//	autosize,				 !- Supply Air Maximum Flow Rate {m3/s}
	fprintf(fp, "\t ,\n");//	,						 !- Zone Heating Sizing Factor
	fprintf(fp, "\t ,\n");//	,						 !- Zone Cooling Sizing Factor
	fprintf(fp, "\t Flow/Person,\n");//	Flow/Person,			 !- Outdoor Air Method
	fprintf(fp, "\t 0.00944,\n");//	0.00944,				 !- Outdoor Air Flow Rate per Person {m3/s}
	fprintf(fp, "\t ,\n");//	,						 !- Outdoor Air Flow Rate per Zone Floor Area {m3/s-m2}
	fprintf(fp, "\t ,\n");//	,						 !- Outdoor Air Flow Rate per Zone {m3/s}
	fprintf(fp, "\t ,\n");//	,						 !- Supply Plenum Name
	fprintf(fp, "\t ,\n");//	,						 !- Return Plenum Name
	fprintf(fp, "\t None,\n");//	None,					 !- Baseboard Heating Type
	fprintf(fp, "\t ,\n");//	,						 !- Baseboard Heating Availability Schedule Name
	fprintf(fp, "\t autosize;\n");//	autosize;				 !- Baseboard Heating Capacity {W}
	fprintf(fp, "\n\n");

    return 0;

}

/*
Schedule:Compact,
    Office HVAC Mixed,       !- Name
    on/off,                  !- Schedule Type Limits Name
    Through: 12/31,          !- Field 1
    For: Weekdays,           !- Field 2
    Until: 8:00,             !- Field 3
    0,                       !- Field 4
    Until: 12:00,            !- Field 5
    1,                       !- Field 6
    Until: 14:00,            !- Field 7
    0,                       !- Field 8
    Until: 18:00,            !- Field 9
    1,                       !- Field 10
    Until: 24:00,            !- Field 11
    0,                       !- Field 12
    For: AllOtherDays,       !- Field 13
    Until 24:00,             !- Field 14
     0;                      !- Field 15


*/
int RoomInfo::generateHVACSchedule(FILE* fp, int flagSchedule)
{
fprintf(fp, "Schedule:Compact,\n");//Schedule:Compact,
	fprintf(fp, "\t HVAC_Schedule_%s,\n", m_roomName);//	Office HVAC Mixed,		 !- Name
	fprintf(fp, "\t on/off,\n");//	on/off, 				 !- Schedule Type Limits Name
	fprintf(fp, "\t Through: 12/31,\n");//	Through: 12/31, 		 !- Field 1
	fprintf(fp, "\t For: AllDays,\n");//	For: AllDays,			 !- Field 2

	//If no meeting, return a null schedule
	if(0 == m_scheduledMeetingNum)
	{
		fprintf(fp, "\t Until 24:00,\n");
		fprintf(fp, "\t 0;\n");//	 0; 					 !- Field 15
		fprintf(fp, "\n\n");
		return 0;
	}


	#define DUMMY_MEETING_LENGTH 30
	int first_start_time = m_meetingSchedule[0]->startTime();
	int st_h = (first_start_time - DUMMY_MEETING_LENGTH)/60;
	int st_m = (first_start_time- DUMMY_MEETING_LENGTH)%60;	
	int first_st_h = (first_start_time)/60;
	int first_st_m = (first_start_time)%60;

	//--------------------------Schedule dummy--------------------
	if(flagSchedule == SCHEDULE_DUMMY)
	{
		fprintf(fp, "\t Until: %2d:%02d,\n", st_h, st_m);
		fprintf(fp, "\t 0,\n");
		fprintf(fp, "\t Until: %2d:%02d,\n", first_st_h, first_st_m);
		fprintf(fp, "\t 1,\n");
	}

	//--------------------------Schedule without recooling--------------------
	if(flagSchedule == SCHEDULE_WITHOUT_RECOOL)
	{
		fprintf(fp, "\t Until: %2d:%02d,\n", st_h, st_m);
		fprintf(fp, "\t 0,\n");
		int total_meeting_length = 0;
		MeetingInfo_ptr current_meeting;
		int total_end_time;
		int total_et_h;
		int total_et_m;
		for(int i=0; i<m_scheduledMeetingNum; i++)
		{
			current_meeting = m_meetingSchedule[i];
			total_meeting_length += current_meeting->endTime()-current_meeting->startTime();
		}
		total_end_time = first_start_time+total_meeting_length;
		total_et_h = total_end_time/60;
		total_et_m = total_end_time%60;
		if(total_et_h>24)
			return 0;
		fprintf(fp, "\t Until: %2d:%02d,\n", total_et_h, total_et_m);
		fprintf(fp, "\t 1,\n");
	}

	//--------------------------Schedule pure-----------------------
	if(flagSchedule == SCHEDULE_PURE)
	{
		MeetingInfo_ptr cur_meeting;
		int cur_start_time, cur_end_time;
		int cur_st_h, cur_st_m, cur_et_h, cur_et_m;
		int last_meeting_end_time = 0;
		//Add meeting schedule
		for(int i = 0; i <m_scheduledMeetingNum; i++)
		{
			cur_meeting = m_meetingSchedule[i];
			cur_start_time = cur_meeting->startTime();
			cur_st_h = cur_start_time/60;
			cur_st_m = cur_start_time%60;

			cur_end_time = cur_meeting->endTime();
			cur_et_h = cur_end_time/60;
			cur_et_m = cur_end_time%60;
			
			//If there is a gap, shutdown the HVAC
			if(last_meeting_end_time < cur_start_time)
			{
				fprintf(fp, "\t Until: %2d:%02d,\n", cur_st_h, cur_st_m);
				fprintf(fp, "\t 0,\n");
			}
			fprintf(fp, "\t Until: %2d:%02d,\n", cur_et_h, cur_et_m);
			fprintf(fp, "\t 1,\n");

			last_meeting_end_time = cur_end_time;
		}
	}
	fprintf(fp, "\t Until 24:00,\n");
	fprintf(fp, "\t 0;\n");//	 0; 					 !- Field 15
	fprintf(fp, "\n\n");

	return 0;
}

