#include "stdio.h"


#include "GroupInfoManager.h"
#include "DPScheduleAlg.h"

#define INPUT_FILE "scheduleResult.txt"
#define OUTDOOR_TEMP_FILE "outdoorTemp.txt"


#define GROUP_NUM 8
#define DEFAULT_GROUP 7
float g_group_room_size[GROUP_NUM][3] = {{4, 5, 3},{8,5,3},{6,10,3},{8,10,3},{10,10,3.3},{10,15,4},{15,14,5},{15,20,6}};
int g_group_room_cap[GROUP_NUM] = {20, 40,60,80,100, 150, 200, 300};
float g_group_Power[GROUP_NUM] = {2000, 4000, 6000, 8000, 12000, 18000, 25000, 35000};


int FileFilter(GroupInfoManager_ptr* gm, int groupNum, const char* file_name, int forRaw);

int scheduleAlgorithm(GroupInfoManager_ptr gm, const char* operationStr)
{
	if(0==strcmp(operationStr, "recoverRawSchedule"))
	{
		return gm->recoverRawSchedule();
	}

	if(0==strcmp(operationStr, "energyAwareFacilitySchedule"))
	{
		return gm->energyAwareFacilitySchedule();
	}
	
	if(0==strcmp(operationStr, "optimalSchedule"))
	{
		return gm->optimalSchedule();
	}
	
	if(0==strcmp(operationStr, "timeUnrestrictedFacilitySchedule"))
	{
		return gm->timeUnrestrictedFacilitySchedule();
	}
	
	if(0==strcmp(operationStr, "timeUnrestrictedSchedule"))
	{
		return gm->timeUnrestrictedSchedule();
	}
	return -1;
}

int schedule(const char* inputFile, const char* operationStr, int group_num, FILE* fpEnergyFile)
{
		GroupInfoManager_ptr gm[GROUP_NUM];
		for(int i = 0; i<GROUP_NUM; i++)
			gm[i]= new GroupInfoManager();
		
		int forRaw = 0;
		if((0==strcmp(operationStr, "recoverRawSchedule"))||(0==strcmp(operationStr, "compressRawSchedule")))
			forRaw = 1;
		if (0==FileFilter(gm, group_num, inputFile, forRaw))
			return -1;
	
		const char* name_tail[7] = {"_Zone.txt","_Surface.txt","_HVAC.txt","_ZoneHVACmap.txt","_Schedule.txt","_Schedule_dummy.txt","_Schedule_wor.txt"};
		
		char filename[7][128] = {"","","","",""};
		for(int i = 0; i<7; i++)
		{
			strcat(filename[i],operationStr);
			strcat(filename[i],name_tail[i]);
		}
		FILE* fp_Zone = fopen(filename[0],"w");
		FILE* fp_Surface= fopen(filename[1],"w");
		FILE* fp_HVAC= fopen(filename[2],"w");
		FILE* fp_ZoneHVACmap= fopen(filename[3],"w");
		FILE* fp_Schedule= fopen(filename[4],"w");
		FILE* fp_Schedule_dummy = fopen(filename[5],"w");
		FILE* fp_Schedule_wor = fopen(filename[6],"w");
	

		//Main process
		float energy_consumption, min_energy_consumption, max_energy_consumption;
		energy_consumption = 0;
		min_energy_consumption = 0;
		max_energy_consumption = 0;

		float origin_Arr[3]={0,0,0};
		//Check whether is for uniform
		if(group_num != 1)
		{
			int unscheduled_num;
			for(int i=0; i<group_num;i++)
			{
				gm[i]->sort();
				scheduleAlgorithm(gm[i], operationStr);
				gm[i]->generateEnergyPlusInput(origin_Arr,fp_Zone,fp_Surface,fp_HVAC,fp_ZoneHVACmap,fp_Schedule, fp_Schedule_dummy, fp_Schedule_wor);
				origin_Arr[0]+=g_group_room_size[i][0];
				unscheduled_num = gm[i]->unscheduledMeetingNum();
				MeetingInfo_ptr* unscheduled_meetings = gm[i]->unscheduledMeeting();
				if(i !=(group_num-1))
				{
					for(int j=0; j< unscheduled_num; j++)
					{
						gm[i+1]->addUnscheduledMeeting(unscheduled_meetings[j]);
					}
				}
				energy_consumption += gm[i]->calculateEnergy();
				min_energy_consumption += gm[i]->calculateMinEnergy();
				max_energy_consumption += gm[i]->calculateMaxEnergy();
				
			}
			printf("Unscheduled meeting %d\n", unscheduled_num);
		}
		else
		{
			gm[DEFAULT_GROUP]->sort();
			scheduleAlgorithm(gm[DEFAULT_GROUP], operationStr);
			gm[DEFAULT_GROUP]->generateEnergyPlusInput(origin_Arr,fp_Zone,fp_Surface,fp_HVAC,fp_ZoneHVACmap,fp_Schedule, fp_Schedule_dummy, fp_Schedule_wor);
			energy_consumption = gm[DEFAULT_GROUP]->calculateEnergy();
			min_energy_consumption = gm[DEFAULT_GROUP]->calculateMinEnergy();
		}
		printf("%s %f, mini %f max: %f\n", operationStr, energy_consumption, min_energy_consumption, max_energy_consumption);
		fprintf(fpEnergyFile,"%f\t%f\n",energy_consumption, min_energy_consumption);
	
	for(int i = 0; i<GROUP_NUM; i++)
	{
		delete gm[i];
	}
	fclose(fp_Zone);
	fclose(fp_Surface);
	fclose(fp_HVAC);
	fclose(fp_ZoneHVACmap);
	fclose(fp_Schedule);
	
	return 0;
}

int main(int argc, char** argv)
{
	g_outdoorTemp= new OutdoorTemperature();
	g_outdoorTemp->init(OUTDOOR_TEMP_FILE,2);

	int group_num = GROUP_NUM; // 1, if uniform
	//int group_num = 1; // 1, if uniform
	FILE* fp_energy_file = fopen("energy_consumption.txt","w");
	schedule(INPUT_FILE, "recoverRawSchedule", group_num, fp_energy_file);
	schedule(INPUT_FILE, "energyAwareFacilitySchedule", group_num, fp_energy_file);
	//schedule(INPUT_FILE, "optimalSchedule", group_num, fp_energy_file);
	schedule(INPUT_FILE, "timeUnrestrictedFacilitySchedule", group_num, fp_energy_file);	
	//schedule(INPUT_FILE, "timeUnrestrictedSchedule", group_num, fp_energy_file);
	fclose(fp_energy_file);
	return 0;

}

int FileFilter(GroupInfoManager_ptr* gm, int groupNum, const char* file_name,int forRaw)
{
	
	FILE* fp;
	fp = fopen(file_name, "r");
	if(fp == NULL)
	{
		printf("Can not open file %s", file_name);
		return 0;
	}

	char classCode[20];
	char subjectCode[20];
	int capacity_req, start_week, end_week, step;
	char week_day[10];
	int start_time_h,start_time_m, end_time_h, end_time_m;
	char room_type[10];
	char room_num[20];
	int  room_capacity;
	int	 meeting_facility_req;
	int  room_facility;
	int  ret;
	int index =0;
	MeetingInfo meeting_info;
	RoomInfo room_info;
	while(1)
	{
		ret = fscanf(fp, "%s%s%d%d%d%d%s%d:%d%d:%d%s%s%d%d%d\n",
			classCode, subjectCode, &capacity_req, &start_week, &end_week, &step, week_day, &start_time_h,&start_time_m, &end_time_h, &end_time_m, room_type, room_num, &room_capacity, &meeting_facility_req, &room_facility);
		if(ret != 16)
			break;
		room_info.roomName(room_num);

		//Select capacity group
		int suitable_room_index = DEFAULT_GROUP;
		int suitable_meeting_index = DEFAULT_GROUP;
		if(groupNum > 1)
		{
			int i;
			for(i= 0; i< groupNum; i++)
			{
				if(room_capacity <= g_group_room_cap[i])
				{
					suitable_room_index = i;
					break;
				}
			}
			for(i= 0; i< groupNum; i++)
			{
				if(capacity_req <= g_group_room_cap[i])
				{
					suitable_meeting_index = i;
					break;
				}
			}
			if(forRaw == 1)
			{
				suitable_meeting_index = suitable_room_index;
			}
			if(i == groupNum)
				continue;
		}

		room_info.facility(room_facility);
		room_info.capacity(g_group_room_cap[suitable_room_index]);
		room_info.roomSizeAndACPower(g_group_room_size[suitable_room_index][0],g_group_room_size[suitable_room_index][1],g_group_room_size[suitable_room_index][2], g_group_Power[suitable_room_index]);

		RoomInfo_ptr selected_room = gm[suitable_room_index]->addRoomInfo(room_info);


		meeting_info.meetingName(subjectCode);
		meeting_info.startTime(start_time_h*60+start_time_m);
		meeting_info.endTime(end_time_h*60+end_time_m);
		meeting_info.capacityRequirement(capacity_req);
		meeting_info.facilityRequirement(meeting_facility_req);
		meeting_info.roomName(room_num);

		// If have conflict, drop the meeting.
		if(-1 == selected_room->isMeetingConflict(&meeting_info))
			continue;

		
		MeetingInfo_ptr added_meeting = gm[suitable_meeting_index]->addMeetingInfo(meeting_info);
		selected_room->putMeeting(added_meeting);
		
		index++;
		if(ret>= 0)
			continue;
	}
	fclose(fp);

	if(groupNum == 1)
		gm[DEFAULT_GROUP]->resetSchedule();
	else
	for(int i=0; i<groupNum; i++)
	{
		gm[i]->resetSchedule();
	}
	
	return 1;
}
